@@ -3,8 +3,8 @@ import warnings
 from braintree.resource import Resource
 from braintree.address import Address
 from braintree.configuration import Configuration
-from braintree.transparent_redirect import TransparentRedirect
 from braintree.credit_card_verification import CreditCardVerification
+from enum import Enum
 
 class CreditCard(Resource):
     """
@@ -28,16 +28,18 @@ class CreditCard(Resource):
                 "postal_code": "60606",
                 "region": "IL",
                 "country_name": "United States of America"
+                "phone_number": "312-123-4567"
             },
             "options": {
-                "verify_card": True
+                "verify_card": True,
+                "verification_amount": "2.00"
             }
         })
 
         print(result.credit_card.token)
         print(result.credit_card.masked_number)
 
-    For more information on CreditCards, see https://www.braintreepayments.com/docs/python/credit_cards/create
+    For more information on CreditCards, see https://developer.paypal.com/braintree/docs/reference/request/credit-card/create/python
 
     """
     class CardType(object):
@@ -49,8 +51,13 @@ class CreditCard(Resource):
         * Braintree.CreditCard.ChinaUnionPay
         * Braintree.CreditCard.DinersClubInternational
         * Braintree.CreditCard.Discover
+        * Braintree.CreditCard.Electron
+        * Braintree.CreditCard.Elo
+        * Braintree.CreditCard.Hiper
+        * Braintree.CreditCard.Hipercard
         * Braintree.CreditCard.JCB
         * Braintree.CreditCard.Laser
+        * Braintree.CreditCard.UK_Maestro
         * Braintree.CreditCard.Maestro
         * Braintree.CreditCard.MasterCard
         * Braintree.CreditCard.Solo
@@ -64,8 +71,13 @@ class CreditCard(Resource):
         ChinaUnionPay = "China UnionPay"
         DinersClubInternational = "Diners Club"
         Discover = "Discover"
+        Electron = "Electron"
+        Elo = "Elo"
+        Hiper = "Hiper"
+        Hipercard = "Hipercard"
         JCB = "JCB"
         Laser = "Laser"
+        UK_Maestro = "UK Maestro"
         Maestro = "Maestro"
         MasterCard = "MasterCard"
         Solo = "Solo"
@@ -84,6 +96,7 @@ class CreditCard(Resource):
         International = "international"
         US = "us"
 
+    # NEXT_MAJOR_VERSION this can be an enum! they were added as of python 3.4 and we support 3.5+
     class CardTypeIndicator(object):
         """
         Constants representing the three states for the card type indicator attributes
@@ -96,23 +109,30 @@ class CreditCard(Resource):
         No = "No"
         Unknown = "Unknown"
 
-    Commercial = DurbinRegulated = Debit = Healthcare = \
-            CountryOfIssuance = IssuingBank = Payroll = Prepaid = CardTypeIndicator
+    class DebitNetwork(Enum):
+        """
+        Constants representing the debit networks used for processing a pinless debit transaction
+
+        * braintree.CreditCard.DebitNetwork.Accel
+        * braintree.CreditCard.DebitNetwork.Maestro
+        * braintree.CreditCard.DebitNetwork.Nyce
+        * braintree.CreditCard.DebitNetwork.Pulse
+        * braintree.CreditCard.DebitNetwork.Star
+        * braintree.CreditCard.DebitNetwork.Star_Access
+        """
+        Accel = "ACCEL"
+        Maestro= "MAESTRO"
+        Nyce = "NYCE"
+        Pulse = "PULSE"
+        Star = "STAR"
+        Star_Access = "STAR_ACCESS"
+
+    Commercial = CountryOfIssuance = Debit = DurbinRegulated = \
+            Business = Consumer = Corporate = Purchase = \
+            Healthcare = IssuingBank = Payroll = Prepaid = PrepaidReloadable = ProductId = CardTypeIndicator
 
     @staticmethod
-    def confirm_transparent_redirect(query_string):
-        """
-        Confirms a transparent redirect request. It expects the query string from the
-        redirect request. The query string should _not_ include the leading "?" character. ::
-
-            result = braintree.CreditCard.confirm_transparent_redirect_request("foo=bar&id=12345")
-        """
-
-        warnings.warn("Please use TransparentRedirect.confirm instead", DeprecationWarning)
-        return Configuration.gateway().credit_card.confirm_transparent_redirect(query_string)
-
-    @staticmethod
-    def create(params={}):
+    def create(params=None):
         """
         Create a CreditCard.
 
@@ -124,11 +144,12 @@ class CreditCard(Resource):
             })
 
         """
-
+        if params is None:
+            params = {}
         return Configuration.gateway().credit_card.create(params)
 
     @staticmethod
-    def update(credit_card_token, params={}):
+    def update(credit_card_token, params=None):
         """
         Update an existing CreditCard
 
@@ -139,7 +160,8 @@ class CreditCard(Resource):
             })
 
         """
-
+        if params is None:
+            params = {}
         return Configuration.gateway().credit_card.update(credit_card_token, params)
 
     @staticmethod
@@ -176,21 +198,6 @@ class CreditCard(Resource):
         """
         return Configuration.gateway().credit_card.find(credit_card_token)
 
-    @staticmethod
-    def forward(credit_card_token, receiving_merchant_id):
-        """
-        Create a nonce for a credit card in your Braintree vault that can be used by another Braintree merchant.
-
-        A credit card token and a receiving merchant ID are required:
-
-            result = braintree.CreditCard.forward(
-                credit_card.token,
-                "another_merchant_public_id"
-            })
-        """
-
-        return Configuration.gateway().credit_card.forward(credit_card_token, receiving_merchant_id)
-
     @staticmethod
     def from_nonce(nonce):
         """
@@ -213,22 +220,74 @@ class CreditCard(Resource):
     @staticmethod
     def signature(type):
         billing_address_params = [
-            "company", "country_code_alpha2", "country_code_alpha3", "country_code_numeric", "country_name",
-            "extended_address", "first_name", "last_name", "locality", "postal_code", "region", "street_address"
+            "company",
+            "country_code_alpha2",
+            "country_code_alpha3",
+            "country_code_numeric",
+            "country_name",
+            "extended_address",
+            "first_name",
+            "last_name",
+            "locality",
+            "postal_code",
+            "region",
+            "street_address",
+            "phone_number"
+        ]
+
+        options = [
+            "account_information_inquiry",
+            "fail_on_duplicate_payment_method",
+            "fail_on_duplicate_payment_method_for_customer",
+            "make_default",
+            "skip_advanced_fraud_checking",
+            "venmo_sdk_session",  # NEXT_MJOR_VERSION remove venmo_sdk_session
+            "verification_account_type",
+            "verification_amount",
+            "verification_merchant_account_id",
+            "verify_card",
+            {
+                "adyen":[
+                    "overwrite_brand",
+                    "selected_brand"
+                ]
+            }
+        ]
+
+        three_d_secure_pass_thru = [
+            "cavv",
+            "ds_transaction_id",
+            "eci_flag",
+            "three_d_secure_version",
+            "xid"
         ]
-        options = ["make_default", "verification_merchant_account_id", "verify_card", "verification_amount", "venmo_sdk_session"]
 
         signature = [
-            "billing_address_id", "cardholder_name", "cvv", "expiration_date", "expiration_month", "expiration_year",
-            "device_session_id", "fraud_merchant_id", "number", "token", "venmo_sdk_payment_method_code", "device_data",
+            "billing_address_id",
+            "cardholder_name",
+            "cvv",
+            "expiration_date",
+            "expiration_month",
+            "expiration_year",
+            "number",
+            "token",
+            "venmo_sdk_payment_method_code", # NEXT_MJOR_VERSION remove venmo_sdk_payment_method_code
+            "device_data",
             "payment_method_nonce",
-            {"billing_address": billing_address_params},
-            {"options": options}
+            "device_session_id", "fraud_merchant_id", # NEXT_MAJOR_VERSION remove device_session_id and fraud_merchant_id
+            {
+                "billing_address": billing_address_params
+            },
+            {
+                "options": options
+            },
+            {
+                "three_d_secure_pass_thru": three_d_secure_pass_thru
+            }
         ]
 
         if type == "create":
             signature.append("customer_id")
-            options.append("fail_on_duplicate_payment_method")
         elif type == "update":
             billing_address_params.append({"options": ["update_existing"]})
         elif type == "update_via_customer":
@@ -239,38 +298,6 @@ class CreditCard(Resource):
 
         return signature
 
-    @staticmethod
-    def transparent_redirect_create_url():
-        """
-        Returns the url to use for creating CreditCards through transparent redirect.
-        """
-        warnings.warn("Please use TransparentRedirect.url instead", DeprecationWarning)
-        return Configuration.gateway().credit_card.transparent_redirect_create_url()
-
-    @staticmethod
-    def tr_data_for_create(tr_data, redirect_url):
-        """
-        Builds tr_data for CreditCard creation.
-        """
-
-        return Configuration.gateway().credit_card.tr_data_for_create(tr_data, redirect_url)
-
-    @staticmethod
-    def tr_data_for_update(tr_data, redirect_url):
-        """
-        Builds tr_data for CreditCard updating.
-        """
-
-        return Configuration.gateway().credit_card.tr_data_for_update(tr_data, redirect_url)
-
-    @staticmethod
-    def transparent_redirect_update_url():
-        """
-        Returns the url to be used for updating CreditCards through transparent redirect.
-        """
-        warnings.warn("Please use TransparentRedirect.url instead", DeprecationWarning)
-        return Configuration.gateway().credit_card.transparent_redirect_update_url()
-
     def __init__(self, gateway, attributes):
         Resource.__init__(self, gateway, attributes)
         self.is_expired = self.expired
@@ -289,6 +316,8 @@ class CreditCard(Resource):
 
     @property
     def expiration_date(self):
+        if not self.expiration_month or not self.expiration_year:
+            return None
         return self.expiration_month + "/" + self.expiration_year
 
     @property
@@ -296,5 +325,7 @@ class CreditCard(Resource):
         """
         Returns the masked number of the CreditCard.
         """
-        return self.bin + "******" + self.last_4
-
+        bin = self.bin_extended if hasattr(self, "bin_extended") else self.bin
+        mask_length = 16 - len(bin) - len(self.last_4)
+        mask = "*" * mask_length
+        return bin + mask + self.last_4
